@@ -730,125 +730,6 @@ found:
 	do_dialog(term, dlg, getml(dlg, NULL));
 }
 
-time_t parse_http_date(unsigned char *date)	/* this functions is bad !!! */
-{
-#ifdef HAVE_MKTIME
-	static unsigned char *months[12] = {
-		cast_uchar "Jan",
-		cast_uchar "Feb",
-		cast_uchar "Mar",
-		cast_uchar "Apr",
-		cast_uchar "May",
-		cast_uchar "Jun",
-		cast_uchar "Jul",
-		cast_uchar "Aug",
-		cast_uchar "Sep",
-		cast_uchar "Oct",
-		cast_uchar "Nov",
-		cast_uchar "Dec"
-	};
-
-	time_t t = 0;
-	/* Mon, 03 Jan 2000 21:29:33 GMT */
-	int y;
-	struct tm tm;
-	memset(&tm, 0, sizeof(struct tm));
-
-	date = cast_uchar strchr(cast_const_char date, ' ');
-	if (!date) return 0;
-	date++;
-	if (*date >= '0' && *date <= '9') {
-			/* Sun, 06 Nov 1994 08:49:37 GMT */
-			/* Sunday, 06-Nov-94 08:49:37 GMT */
-		y = 0;
-		if (date[0] < '0' || date[0] > '9') return 0;
-		if (date[1] < '0' || date[1] > '9') return 0;
-		tm.tm_mday = (date[0] - '0') * 10 + date[1] - '0';
-		date += 2;
-		if (*date != ' ' && *date != '-') return 0;
-		date += 1;
-		for (tm.tm_mon = 0; tm.tm_mon < 12; tm.tm_mon++)
-			if (!casecmp(date, months[tm.tm_mon], 3)) goto f1;
-		return 0;
-		f1:
-		date += 3;
-		if (*date == ' ') {
-				/* Sun, 06 Nov 1994 08:49:37 GMT */
-			date++;
-			if (date[0] < '0' || date[0] > '9') return 0;
-			if (date[1] < '0' || date[1] > '9') return 0;
-			if (date[2] < '0' || date[2] > '9') return 0;
-			if (date[3] < '0' || date[3] > '9') return 0;
-			tm.tm_year = (date[0] - '0') * 1000 + (date[1] - '0') * 100 + (date[2] - '0') * 10 + date[3] - '0' - 1900;
-			date += 4;
-		} else if (*date == '-') {
-				/* Sunday, 06-Nov-94 08:49:37 GMT */
-			date++;
-			if (date[0] < '0' || date[0] > '9') return 0;
-			if (date[1] < '0' || date[1] > '9') return 0;
-			tm.tm_year = (date[0] >= '7' ? 1900 : 2000) + (date[0] - '0') * 10 + date[1] - '0' - 1900;
-			date += 2;
-		} else return 0;
-		if (*date != ' ') return 0;
-		date++;
-	} else {
-			/* Sun Nov  6 08:49:37 1994 */
-		y = 1;
-		for (tm.tm_mon = 0; tm.tm_mon < 12; tm.tm_mon++)
-			if (!casecmp(date, months[tm.tm_mon], 3)) goto f2;
-		return 0;
-		f2:
-		date += 3;
-		while (*date == ' ') date++;
-		if (date[0] < '0' || date[0] > '9') return 0;
-		tm.tm_mday = date[0] - '0';
-		date++;
-		if (*date != ' ') {
-			if (date[0] < '0' || date[0] > '9') return 0;
-			tm.tm_mday = tm.tm_mday * 10 + date[0] - '0';
-			date++;
-		}
-		if (*date != ' ') return 0;
-		date++;
-	}
-
-	if (date[0] < '0' || date[0] > '9') return 0;
-	if (date[1] < '0' || date[1] > '9') return 0;
-	tm.tm_hour = (date[0] - '0') * 10 + date[1] - '0';
-	date += 2;
-	if (*date != ':') return 0;
-	date++;
-	if (date[0] < '0' || date[0] > '9') return 0;
-	if (date[1] < '0' || date[1] > '9') return 0;
-	tm.tm_min = (date[0] - '0') * 10 + date[1] - '0';
-	date += 2;
-	if (*date != ':') return 0;
-	date++;
-	if (date[0] < '0' || date[0] > '9') return 0;
-	if (date[1] < '0' || date[1] > '9') return 0;
-	tm.tm_sec = (date[0] - '0') * 10 + date[1] - '0';
-	date += 2;
-	if (y) {
-		if (*date != ' ') return 0;
-		date++;
-		if (date[0] < '0' || date[0] > '9') return 0;
-		if (date[1] < '0' || date[1] > '9') return 0;
-		if (date[2] < '0' || date[2] > '9') return 0;
-		if (date[3] < '0' || date[3] > '9') return 0;
-		tm.tm_year = (date[0] - '0') * 1000 + (date[1] - '0') * 100 + (date[2] - '0') * 10 + date[3] - '0' - 1900;
-		date += 4;
-	}
-	if (*date != ' ' && *date) return 0;
-
-	t = mktime(&tm);
-	if (t == (time_t) -1) return 0;
-	return t;
-#else
-	return 0;
-#endif
-}
-
-
 static void download_file_error(struct download *down, int err)
 {
 	struct session *ses = get_download_ses(down);
@@ -940,12 +821,8 @@ static void download_data(struct status *stat, void *down_)
 				prev_down_url = down->url;
 				down->url = u;
 				down->stat.state = S_WAIT_REDIR;
-				if (down->win) {
-					struct links_event ev = { EV_REDRAW, 0, 0, 0 };
-					ev.x = down->win->term->x;
-					ev.y = down->win->term->y;
-					down->win->handler(down->win, &ev, 0);
-				}
+				if (down->win)
+					redraw_window(down->win);
 				cache = NC_CACHE;
 				if (!strcmp(cast_const_char down->url, cast_const_char prev_down_url) || down->redirect_cnt >= MAX_CACHED_REDIRECTS) cache = NC_RELOAD;
 				allow_flags = get_allow_flags(prev_down_url);
@@ -1050,12 +927,8 @@ have_frag:
 		abort_download(down);
 		return;
 	}
-	if (down->win) {
-		struct links_event ev = { EV_REDRAW, 0, 0, 0 };
-		ev.x = down->win->term->x;
-		ev.y = down->win->term->y;
-		down->win->handler(down->win, &ev, 0);
-	}
+	if (down->win)
+		redraw_window(down->win);
 }
 
 unsigned char *translate_download_file(unsigned char *fi)
@@ -1822,22 +1695,25 @@ void html_interpret_recursive(struct f_data_c *f)
 struct additional_file *request_additional_file(struct f_data *f, unsigned char *url_)
 {
 	size_t sl;
+	unsigned h;
 	struct additional_file *af;
-	struct list_head *laf;
 	unsigned char *u, *url;
 	url = stracpy(url_);
 	if ((u = extract_position(url))) mem_free(u);
 	if (!f->af) {
 		if (!(f->af = f->fd->af)) {
-			f->af = f->fd->af = mem_alloc(sizeof(struct additional_files));
+			f->af = f->fd->af = mem_calloc(sizeof(struct additional_files));
 			f->af->refcount = 1;
 			init_list(f->af->af);
 		}
 		f->af->refcount++;
 	}
-	foreach(struct additional_file, af, laf, f->af->af) if (!strcmp(cast_const_char af->url, cast_const_char url)) {
-		mem_free(url);
-		return af;
+	h = hash_string(url) % AF_HASH_SIZE;
+	for (af = f->af->hash[h]; af; af = af->hash_fwdlink) {
+		if (!strcmp(cast_const_char af->url, cast_const_char url)) {
+			mem_free(url);
+			return af;
+		}
 	}
 	sl = strlen(cast_const_char url);
 	if (sl > MAXINT - sizeof(struct additional_file)) overalloc();
@@ -1852,6 +1728,8 @@ struct additional_file *request_additional_file(struct f_data *f, unsigned char
 	af->need_reparse = 0;
 	af->unknown_image_size = 0;
 	add_to_list(f->af->af, af);
+	af->hash_fwdlink = f->af->hash[h];
+	f->af->hash[h] = af;
 	mem_free(url);
 	return af;
 }
@@ -1863,17 +1741,20 @@ static void copy_additional_files(struct additional_files **a)
 	struct list_head *laf;
 	if (!*a || (*a)->refcount == 1) return;
 	(*a)->refcount--;
-	afs = mem_alloc(sizeof(struct additional_files));
+	afs = mem_calloc(sizeof(struct additional_files));
 	afs->refcount = 1;
 	init_list(afs->af);
 	foreachback(struct additional_file, af, laf, (*a)->af) {
 		struct additional_file *afc;
+		unsigned h = hash_string(af->url) % AF_HASH_SIZE;
 		size_t sl = strlen(cast_const_char af->url);
 		if (sl > MAXINT - sizeof(struct additional_file)) overalloc();
 		afc = mem_alloc(sizeof(struct additional_file) + sl);
 		memcpy(afc, af, sizeof(struct additional_file) + sl);
 		if (af->rq) clone_object(af->rq, &afc->rq);
 		add_to_list(afs->af, afc);
+		afc->hash_fwdlink = afs->hash[h];
+		afs->hash[h] = afc;
 	}
 	*a = afs;
 }
